/* solitaire.c
 * 
 * author: skalar
 * creation date: i forgor :skull:
 * completion date: 04/sept/2021
 * description: shitty terminal solitaire game
 *
 * dependencies:
 * 		manstr.h <attainable from xnopyt>
 *
 * instructions to compile:
 * 		gcc solitaire.c -lpthread -lm
 * 		clang solitiare.c -lpthread -lm
 */

/* preprocessor defines */
//#define DEBUG_MODE //uncomment to enable debug mode

#define PROJ_NAME "solitaire"
#define nput(f) (printf("[" PROJ_NAME "] " f))
#define nfputf(f, ...) (printf("[" PROJ_NAME "] " f, __VA_ARGS__))

#define DRAW_STACK -1
#define STACKS_MIN 0
#define STACKS_MAX 6
#define FIN_MIN 7
#define FIN_MAX 10

#define DRAW_NUM 1
#define STACK_NUM 7
#define FIN_NUM 4

#define DRAW_OFFSET_X (4)
#define DRAW_OFFSET_Y (2)
#define STACKS_OFFSET_X (DRAW_OFFSET_X + 4)
#define STACKS_OFFSET_Y (2)
#define STACKS_SEPERATION (2)
#define FIN_OFFSET_X (STACKS_OFFSET_X + (STACK_NUM * STACKS_SEPERATION) + 4)
#define FIN_OFFSET_Y (STACKS_OFFSET_Y + 0)
#define FIN_SEPERATION STACKS_SEPERATION

#define CURSOR_OFFSET_X (1)
#define CURSOR_OFFSET_Y (0)
#define CURSOR_STACKS_MARCH STACKS_SEPERATION

#define HELD_OFFSET_Y 13
#define HELD_SEPERATION 1

/* system includes */
#define _GNU_SOURCE
#include<math.h>
#include<stdio.h>
#include<stdlib.h>
#include<termios.h>
#include<pthread.h>
#include<unistd.h>

/* library includes */
#include<manstr.h>

/* local includes */

/* code */

// types

typedef struct card_t {
	int num;
	enum { 
		spade,
		club,
		heart,
		diamond
	} suit;
	int discovered;
	int deletion;
} card_t;

typedef struct stack_t {
	card_t* cards;
	int numOfCards;
} stack_t;

typedef struct game_t {
	stack_t draw;
	stack_t drawOpen;
	stack_t stacks[7];
	stack_t finish[4];

	/* -1 -> 	 draw stacc
	 * 0 - 6 ->  regular stacks
	 * 7 - 10 -> finish stacks */
	int cursor_stack, cursor_y, cursor_blink, blink;
	stack_t held;
	stack_t* lastStack;

	int moves;

	time_t startTime;
	int drawAmount;
} game_t;

// globals
static struct termios G_originalTermios;

static char* G_color[] = {
	"\033[38;5;245m",
	"\033[38;5;242m",
	"\033[38;5;160m",
	"\033[38;5;197m"
};

static char* G_faceCards[] = {
	"J",
	"Q",
	"K",
};

// funcs

void onExitFunction() {
	printf("\033[?25h");
}

void disableRawMode() {
	tcsetattr(STDIN_FILENO, TCSAFLUSH, &G_originalTermios);
}

void enableRawMode() {
	struct termios raw;
	tcgetattr(STDIN_FILENO, &raw);
	G_originalTermios = raw;
	atexit(disableRawMode);
	raw.c_lflag &= ~(ECHO | ICANON);
	// raw.c_lflag &= ~(ISIG);
	raw.c_cc[VMIN] = 0;
	raw.c_cc[VTIME] = 0;
	tcsetattr(STDIN_FILENO, TCSAFLUSH, &raw);

}

static inline double lerp(double v0, double v1, double in) {
	return v0 + in * (v1 - v0);
}

static inline double ease(double in, double p_pow) {
	if(in > 0.5)
		return pow(in, 1/p_pow);
	else
		return pow(in, p_pow);
}

void seedRandom() {
	FILE* rand = fopen("/dev/urandom", "r");
	int q;
	fread(&q, 1, 4, rand);
	srand(q);
	fclose(rand);
}

stack_t createDeck() {
	card_t* deck = malloc(1); // init deck
	for(int suit = 0; suit < 4; suit++) {
		for(int num = 0; num < 13; num++) {
			card_t newCard = {0};
			newCard.num = num;
			newCard.suit = suit;
			newCard.deletion = 0;
			deck = realloc(deck, sizeof(card_t) * ( num + suit * 13 + 1 ));
			deck[num + suit * 13] = newCard;
		}
	}
	stack_t r_stack = {0};
	r_stack.cards = deck;
	r_stack.numOfCards = 52;
	return r_stack;
}

void shuffleDeck(stack_t p_deck, int p_randomness) {
	card_t tmp;
	for(int i = 0; i < p_randomness; i++) {
		int r = rand() % p_deck.numOfCards;
		int q = rand() % p_deck.numOfCards;
		tmp = p_deck.cards[r];
		p_deck.cards[r] = p_deck.cards[q];
		p_deck.cards[q] = tmp;
	}
}

// get rid of voids in a deck
void flattenStack(stack_t* p_deck) {
	if(p_deck->numOfCards == 1) {
		if(p_deck->cards[0].deletion == 1) {
			p_deck->numOfCards--;
		} return;
	}
	for(int i = 0; i < p_deck->numOfCards; i++) {
		if(p_deck->cards[i].deletion == 1) {
			for(int j = i; j < p_deck->numOfCards; j++) {
				p_deck->cards[j] = p_deck->cards[j + 1];
			}
			p_deck->numOfCards--;
			p_deck->cards = realloc(p_deck->cards, sizeof(card_t) * p_deck->numOfCards);
			i--;
		} else if (i == p_deck->numOfCards - 1 && \
				p_deck->cards[i].deletion == 1) p_deck->numOfCards--;
	}
}

// adds a card to the top of a stack
void addCard(stack_t* p_stack, card_t p_card) {
	p_stack->numOfCards++;
	p_stack->cards = realloc(p_stack->cards, sizeof(card_t) * p_stack->numOfCards);
	p_stack->cards[p_stack->numOfCards - 1] = p_card;
}

game_t createGame() {
	game_t r_game = {0};
	r_game.draw = createDeck();
	shuffleDeck(r_game.draw, 500);

	// zero out decks
	for(int i = 0; i < 4; i++)
		r_game.finish[i] = (stack_t) {0};

	for(int i = 0; i < 7; i++)
		r_game.stacks[i] = (stack_t) {0};

	// move cards from draw stack to placing stacks
	for(int i = 0; i < 7; i++) {
		for(int j = 0; j < i + 1; j++) {
			int r = rand() % r_game.draw.numOfCards;
			//printf("%i %i\n", r, r_game.draw.cards[r].num);
			addCard(&r_game.stacks[i], r_game.draw.cards[r]);
			r_game.draw.cards[r].deletion = 1;
			flattenStack(&r_game.draw);
		}
	}

	for(int i = 0; i < 7; i++) {
		r_game.stacks[i].cards[r_game.stacks[i].numOfCards-1].discovered = 1;
	}

	for(int i = 0; i < r_game.draw.numOfCards; i++) 
		r_game.draw.cards[i].discovered = 1;

	return r_game;
}

void freegame(game_t* p_game) {
	printf("\033[2J");
	exit(0);
}

int compatable(card_t* target, card_t* source) {
	if(target->num - 1 == source->num) {
		if((target->suit == spade || target->suit == club)
				&& (source->suit == heart || source->suit == diamond))
			return 1;
		if((target->suit == heart || target->suit == diamond)
				&& (source->suit == spade || source->suit == club))
			return 1;
		}
	return 0;
}

int finishCompatable(card_t* target, card_t* source) {
	if(target->num + 1 == source->num)
		if(target->suit == source->suit) return 1;
	return 0;
}

void unhide(game_t* p_game) {
	for(int i = 0; i < STACK_NUM; i++)
		if(p_game->stacks[i].numOfCards != 0)
			p_game->stacks[i].cards[p_game->stacks[i].numOfCards-1].discovered |= 1;
	for(int i = 0; i < FIN_NUM; i++)
		if(p_game->finish[i].numOfCards != 0)
			p_game->finish[i].cards[p_game->finish[i]\
				.numOfCards-1].discovered |= 1;
}

void moveStackToStack(stack_t* target, stack_t* source, int target_pos) {
	for(int i = target_pos; i < source->numOfCards; i++) {
		addCard(target, source->cards[i]);
		source->cards[i].deletion = 1;
	}	flattenStack(source);
}

void draw(game_t* p_game);

void behavoir(game_t* p_game) {
	char buf = '\0';
	read(STDIN_FILENO, &buf, 1);
	switch(buf) {
		case '\033':
			read(STDIN_FILENO, &buf, 1);
			read(STDIN_FILENO, &buf, 1);
			switch(buf) {
				case 'A':
					p_game->cursor_y--;
					break;
				case 'B':
					p_game->cursor_y++;
					break;
				case 'C':
					p_game->cursor_stack++;
					break;
				case 'D':
					p_game->cursor_stack--;
					break;
				default: break;
			}
			break;
		case 'n': // pull from draw pile
			if(p_game->held.numOfCards == 0)
			{
				if(p_game->draw.numOfCards != 0) {
					for(int i = 0; i < p_game->drawAmount; i++) {
						if(i >= p_game->draw.numOfCards) break;
						addCard(&p_game->drawOpen, \
								p_game->draw.cards[i]);
						p_game->draw.cards[i].deletion = 1;
					} flattenStack(&p_game->draw);
				} else {
					moveStackToStack(&p_game->draw, &p_game->drawOpen, 0);
				}
				p_game->moves++;
			}
			break;
		case '\n': // picking up / placing down
			/* pick up for draw stack */
			if(p_game->cursor_stack == DRAW_STACK) {
				if(p_game->held.numOfCards == 0 && p_game->drawOpen.numOfCards > 0) {
					addCard(&p_game->held, \
							p_game->drawOpen.cards[p_game->drawOpen.numOfCards - 1]);
					p_game->drawOpen.numOfCards--;
					p_game->lastStack = &p_game->drawOpen;
				}
			}

			/* pick up/drop for standard stacks */
			if(p_game->cursor_stack >= STACKS_MIN && p_game->cursor_stack <= STACKS_MAX) {
				if(p_game->held.numOfCards == 0) { // pick up cards
					if(p_game->stacks[p_game->cursor_stack]\
							.cards[p_game->cursor_y].discovered) {
						moveStackToStack(&p_game->held, \
								&p_game->stacks[p_game->cursor_stack],p_game->cursor_y);
						p_game->lastStack = &p_game->stacks[p_game->cursor_stack];
					}
				} else { // drop cards
					if(p_game->stacks[p_game->cursor_stack].numOfCards == 0) 
					{ //empty deck
						if(p_game->held.cards[0].num == 12){ 
							moveStackToStack(&p_game->stacks[p_game->cursor_stack], \
									&p_game->held, 0);
							p_game->moves++;
						} else // failure, return to last stack
							moveStackToStack(p_game->lastStack, &p_game->held, 0);
					} else { // nonempty deck
						card_t* stackCard = &p_game->stacks[p_game->cursor_stack]\
							.cards[p_game->stacks[p_game->cursor_stack].numOfCards - 1];
						if(compatable(stackCard, &p_game->held.cards[0]) == 1) {
							moveStackToStack(&p_game->stacks[p_game->cursor_stack],\
									&p_game->held, 0);
							p_game->moves++;
						} else // failure, return to last stack
							moveStackToStack(p_game->lastStack, &p_game->held, 0);
					}
				}
			}

			/* pick up/drop for fin stacks */
			if(p_game->cursor_stack >= FIN_MIN && p_game->cursor_stack <= FIN_MAX) {
				int tmp = p_game->cursor_stack;
				p_game->cursor_stack -= FIN_MIN;
				if(p_game->held.numOfCards == 0) { // pick up
					if(p_game->finish[p_game->cursor_stack].numOfCards != 0) {
						moveStackToStack(&p_game->held,\
								&p_game->finish[p_game->cursor_stack],\
								p_game->finish[p_game->cursor_stack].numOfCards - 1);
						p_game->lastStack = &p_game->finish[p_game->cursor_stack];
					}
				} else { // drop cards
					if(p_game->finish[p_game->cursor_stack].numOfCards > 0)
					{ // nonempty stack
						card_t* stackCard = &p_game->finish[p_game->cursor_stack]\
								.cards[p_game->finish[p_game->cursor_stack].numOfCards -1];
						if(finishCompatable(stackCard, \
								&p_game->held.cards[0]) == 1 \
								&& p_game->held.numOfCards == 1) {
							moveStackToStack(&p_game->finish[p_game->cursor_stack],
									&p_game->held, 0);
							p_game->moves++;
						} else { // failure, return to last stack
							moveStackToStack(p_game->lastStack, &p_game->held, 0);
						}
					} else { // empty stack
						if(p_game->held.cards[0].num == 0 \
								&& p_game->held.numOfCards == 1) {
							moveStackToStack(&p_game->finish[p_game->cursor_stack],
									&p_game->held, 0);
							p_game->moves++;
						} else { // failure, return to last stack
							moveStackToStack(p_game->lastStack, &p_game->held, 0);
						}
					} 
				}
				p_game->cursor_stack = tmp;
			}
			break;
		case 'q': // quit game
			freegame(p_game);
	}

	if(buf == '`')
		p_game->cursor_stack = DRAW_STACK;
	if(buf >= '0' && buf <= '9') {
		if(buf != '0')
			p_game->cursor_stack = buf - '0' - 1;
		else
			p_game->cursor_stack = buf - '0' + 9;
	}
	if(buf == '-')
		p_game->cursor_stack = FIN_MAX;

	// fix cursor within all of the stacks
	if(p_game->cursor_stack < DRAW_STACK) 
		p_game->cursor_stack = DRAW_STACK;
	if(p_game->cursor_stack > FIN_MAX)
		p_game->cursor_stack = FIN_MAX;
	if(p_game->cursor_stack == DRAW_STACK) {
		if(p_game->cursor_y < DRAW_OFFSET_Y) p_game->cursor_y = DRAW_OFFSET_Y;
		if(p_game->cursor_y < 0) p_game->cursor_y = 0;
	}

	// stacks
	if(p_game->cursor_stack >= STACKS_MIN && p_game->cursor_stack <= STACKS_MAX) {
		if(p_game->cursor_y > p_game->stacks[p_game->cursor_stack].numOfCards - 1)
			p_game->cursor_y = p_game->stacks[p_game->cursor_stack].numOfCards - 1;
		if(p_game->cursor_y < 0) p_game->cursor_y = 0;
	}

	if(p_game->cursor_stack >= FIN_MIN && p_game->cursor_stack <= FIN_MAX) {
		if(p_game->cursor_y > p_game->finish[p_game->cursor_stack-7].numOfCards-1)
			p_game->cursor_y = p_game->finish[p_game->cursor_stack-7].numOfCards-1;
		if(p_game->cursor_y < 0) p_game->cursor_y = 0;
	}

	if(p_game->held.numOfCards != 0)
		p_game->blink = 1;
	else
		p_game->blink = 0;

	if(buf != 0)
		draw(p_game);

}

void* cursorBlinkThread(void* p_args) {
	game_t* game = p_args;
	
	for(;;) {
		if(game->blink)
			game->cursor_blink ^= 1;
		else
			game->cursor_blink = 1;
		usleep(250000);
	}

	return NULL;
}

void draw(game_t* p_game) {
	mstr_t frame = mstr("");
	astrf(&frame,"\033[2J\033[H\033[;49m");
	// draw drawpile
	if(p_game->draw.numOfCards != 0)
		astrf(&frame,"\033[%i;%iHX", DRAW_OFFSET_Y, DRAW_OFFSET_X);

	if(p_game->drawOpen.numOfCards != 0) {
		astrf(&frame,"\033[%i;%iH", DRAW_OFFSET_Y + 1, DRAW_OFFSET_X);
		astrf(&frame,"%s", G_color[p_game->drawOpen.cards[p_game->drawOpen.numOfCards-1].suit]);
		if(p_game->drawOpen.cards[p_game->drawOpen.numOfCards-1].num < 10)
			astrf(&frame,"%i", p_game->drawOpen.cards[p_game->drawOpen.numOfCards-1].num);
		else
			astrf(&frame,"%s", \
				G_faceCards[p_game->drawOpen.cards[p_game->drawOpen.numOfCards-1].num - 10]);
	}

	// draw stacks

	astrf(&frame,"\033[;49m");

	for(int i = 0; i < 7; i++) {
		astrf(&frame,"\033[%i;%iH\033[A_", STACKS_OFFSET_Y, \
				i * STACKS_SEPERATION + STACKS_OFFSET_X);
		for(int j = 0; j < p_game->stacks[i].numOfCards; j++) {
			astrf(&frame,"\033[%i;%iH", j + STACKS_OFFSET_Y, \
					i * STACKS_SEPERATION+ STACKS_OFFSET_X);
			astrf(&frame,"%s", G_color[p_game->stacks[i].cards[j].suit]);	
#ifndef DEBUG_MODE
			if(p_game->stacks[i].cards[j].discovered == 1)
#else
			if(1)
#endif
			{
				if(p_game->stacks[i].cards[j].num < 10)
					astrf(&frame,"%i", p_game->stacks[i].cards[j].num);
				else
					astrf(&frame,"%s", G_faceCards[p_game->stacks[i].cards[j].num - 10]);
			} else astrf(&frame,"\033[38;5;242mX");
			astrf(&frame,"\033[;49m");
		}
	}

	// draw fin
	for(int i = 0; i < 4; i++) {
		astrf(&frame,"\033[%i;%iH\033[A_", FIN_OFFSET_Y, \
				i * FIN_SEPERATION + FIN_OFFSET_X);
		for(int j = 0; j < p_game->finish[i].numOfCards; j++) {
			astrf(&frame,"\033[%i;%iH", j + FIN_OFFSET_Y, \
					i * FIN_SEPERATION + FIN_OFFSET_X);
			astrf(&frame,"%s", G_color[p_game->finish[i].cards[j].suit]);	
#ifndef DEBUG_MODE
			if(p_game->finish[i].cards[j].discovered == 1)
#else
			if(1)
#endif
			{
				if(p_game->finish[i].cards[j].num < 10)
					astrf(&frame,"%i", p_game->finish[i].cards[j].num);
				else
					astrf(&frame,"%s", G_faceCards[p_game->finish[i].cards[j].num - 10]);
			} else astrf(&frame,"\033[;49mX");
			astrf(&frame,"\033[;49m");
		}
	}

	// print cursor
	astrf(&frame,"\033[%i;%iH", p_game->cursor_y + CURSOR_OFFSET_Y,\
			(p_game->cursor_stack * CURSOR_STACKS_MARCH) + CURSOR_OFFSET_X);
	if(p_game->cursor_blink == 1)
		astrf(&frame,"\033[38;5;249m");
	else
		astrf(&frame,"\033[;49m");
	if(p_game->cursor_stack == DRAW_STACK)
		astrf(&frame,"\033[%i;%iHV", DRAW_OFFSET_Y - 1, DRAW_OFFSET_X);
	if(p_game->cursor_stack >= STACKS_MIN \
			&& p_game->cursor_stack <= STACKS_MAX)
		astrf(&frame,"\033[%i;%iH<", STACKS_OFFSET_Y + p_game->cursor_y,\
				STACKS_OFFSET_X + STACKS_SEPERATION * p_game->cursor_stack \
				+ CURSOR_OFFSET_X);
	if(p_game->cursor_stack >= FIN_MIN \
			&& p_game->cursor_stack <= FIN_MAX)
		astrf(&frame,"\033[%i;%iH>", FIN_OFFSET_Y + p_game->cursor_y,\
				FIN_OFFSET_X + (FIN_SEPERATION * (p_game->cursor_stack - FIN_MIN) - 1));

	int heldHeight = HELD_OFFSET_Y;
	for(int i = 0; i < STACK_NUM; i++)
		if(p_game->stacks[i].numOfCards > heldHeight)
			heldHeight += p_game->stacks[i].numOfCards - HELD_OFFSET_Y;
	astrf(&frame,"\033[%i;3H\033[;49mHELD: ", heldHeight);
	astrf(&frame,"\033[%iD", HELD_SEPERATION);
	for(int i = 0; i < p_game->held.numOfCards; i++)
		if(p_game->held.cards[i].num < 10)
			astrf(&frame,"\033[%iC%s%i", HELD_SEPERATION, \
					G_color[p_game->held.cards[i].suit], p_game->held.cards[i].num);
		else
			astrf(&frame,"\033[%iC%s%s", HELD_SEPERATION, \
					G_color[p_game->held.cards[i].suit], \
					G_faceCards[p_game->held.cards[i].num-10]);

	astrf(&frame,"\033[%i;3H\033[;49mMOVES: %i",  heldHeight + 3, p_game->moves);
	

	astrf(&frame,"\033[;49m");
	astrf(&frame,"\033[%i;%iHcontrols: " , 2, FIN_NUM * FIN_SEPERATION + FIN_OFFSET_X + 2);
	astrf(&frame,"\033[%i;%iHentr: \033[38;5;117mmeth\033[;49m hands" , 3, \
			FIN_NUM * FIN_SEPERATION + FIN_OFFSET_X + 2);
	astrf(&frame,"\033[%i;%iHq: \033[31;49mmurder\033[;49m", \
			4, FIN_NUM * FIN_SEPERATION + FIN_OFFSET_X + 2);
	astrf(&frame,"\033[%i;%iHN: drawpile thyme" , 5, \
			FIN_NUM * FIN_SEPERATION + FIN_OFFSET_X + 2);
	astrf(&frame,"\033[%i;%iH`...-: jump up and get down", 6, \
			FIN_NUM * FIN_SEPERATION + FIN_OFFSET_X + 2);
	astrf(&frame,"\033[%i;%iHidk the rest i kinda forgor :skul", 8,\
			FIN_NUM * FIN_SEPERATION + FIN_OFFSET_X + 2);

	write(0, str(frame), frame.len);
	freeString(&frame);

}

void drawTime(game_t* p_game) {
	mstr_t frame = mstr("");

	int heldHeight = HELD_OFFSET_Y;
	for(int i = 0; i < STACK_NUM; i++)
		if(p_game->stacks[i].numOfCards > heldHeight)
			heldHeight += p_game->stacks[i].numOfCards - HELD_OFFSET_Y;

	int timeLeft = time(NULL) - p_game->startTime;
	int sec = timeLeft % 60, min = timeLeft / 60;
	astrf(&frame,"\033[%i;3H\033[;49mTIME: ", heldHeight + 4);
	(min < 10) ? astrf(&frame,"0%i:", min) : astrf(&frame,"%i:", min);
	(sec < 10) ? astrf(&frame,"0%i", sec) : astrf(&frame,"%i", sec);

	write(0, str(frame), frame.len);
	freeString(&frame);
}

int checkForFinish(game_t* p_game) {
	if(p_game->finish[0].numOfCards == 13
			&& p_game->finish[1].numOfCards == 13
			&& p_game->finish[2].numOfCards == 13
			&& p_game->finish[3].numOfCards == 13)
		return 1;
	else return 0;
}

void win(game_t* p_game) {
	int min, sec;
	time_t endTime = time(NULL);
	int totalTime = endTime - p_game->startTime;
	printf("\033[2J\033[;49m");
	printf("\033[;Hu won.\n");
	printf("ur time: \n\033[32;49m");
	for(int i = 0; i <= totalTime; i++) { 
		sec = (int)(i) % 60;
		min = (i) / 60;
		printf("\r");
		(min < 10) ? printf("0%i:", min) : printf("%i:", min);
		(sec < 10) ? printf("0%i", sec) : printf("%i", sec);
		fflush(stdout);
		usleep(5000 * lerp(0, 1, ease((double)i/totalTime, 1.5)));
	}
	printf("\nur moves: \n\033[32;49m");
	for(int i = 0; i <= p_game->moves; i++) {
		printf("\r%i", i);
		fflush(stdout);
		usleep(10000 * lerp(0, 1, ease((double)i/p_game->moves, 3)));
	}
	usleep(1000000);
	printf("\n");
	fflush(stdout);
	lerp(2, 2, 2);
}

void DEBUG(game_t* p_game);

int main() {
	enableRawMode();
	seedRandom();
	atexit(onExitFunction);

	printf("\033[?25l");

	game_t game = createGame();

	for(;;) {
		printf("\033[;H\033[2J\033[;49mpick ur style: ");
		printf("\n\n [\033[32;49m1\033[;49m] draw 1 \
				\n [\033[33;49m3\033[;49m] draw 3 \
				\n [\033[31;49mq\033[;49m] to \033[31;49mdie");
		char buf = 0;
		read(STDIN_FILENO, &buf, 1);
		if(buf == '1') {
			game.drawAmount = 1;
			break;
		} else if (buf == '3') {
			game.drawAmount = 3;
			break;
		} else if (buf == 'q') {
			printf("\n\033[;49m");
			exit(0);
		}
		fflush(stdout);
		usleep(50000);
	}

	pthread_t cursorBlink;
	pthread_create(&cursorBlink, NULL, cursorBlinkThread, &game);

	game.startTime = time(NULL);

	float deltaTime = 0;
	clock_t performance = clock();

	for(;;) {
		deltaTime = (float)(performance - clock()) / CLOCKS_PER_SEC;
		performance = clock();
		behavoir(&game); // movement/pickup
		draw(&game); // draws the board
		if(game.held.numOfCards == 0) unhide(&game);
		drawTime(&game);
		#ifdef DEBUG_MODE
			DEBUG(&game); // debug function
		#endif
		if(checkForFinish(&game) == 1) {
			win(&game);
			break;
		}
		fflush(stdout);
		if(deltaTime < 1.0/8)
			usleep(((1.0/24) * 1000000) - deltaTime);
	}

	printf("\033[?25h");

	pthread_exit(&cursorBlink);

	printf("Draw: ");
	return 0;
}

void DEBUG(game_t* p_game) {
	printf("\033[15;H");
	printf("LAST MOVEMENT: \n");
	printf("CURSOR Y: %i\n", p_game->cursor_y);
	printf("CURSOR X: %i\n", p_game->cursor_stack);
	printf("DRAW PILE: ");
	for(int i = 0; i < p_game->draw.numOfCards; i++)
		printf("%i ", p_game->draw.cards[i].num);
	printf("\nDRAW NUMCARDS: %i\n", p_game->draw.numOfCards);
	printf("DRAWOPEN PILE: ");
	for(int i = 0; i < p_game->drawOpen.numOfCards; i++) 
		printf("%i ", p_game->drawOpen.cards[i].num);
	printf("\nDRAW NUMCARDS: %i\n", p_game->drawOpen.numOfCards);
	printf("HELD PILE: ");
	for(int i = 0; i < p_game->held.numOfCards; i++)
		printf("%i ", p_game->held.cards[i].num);
	printf("\nHELD NUMCARDS: %i\n", p_game->held.numOfCards);
	printf("STACK NUMS: ");
	for(int i = 0; i < STACK_NUM; i++)
		printf("%i ", p_game->stacks[i].numOfCards);
}
